from __future__ import annotations
from enum import IntEnum

from typing import List


class CellValue(IntEnum):
    NONE = 0

    GROUND_FIRST = 101
    GROUND_SEA = 101
    GROUND_EARTH = 102
    GROUND_LAST = 103

    IMPASSABLE_FIRST = 201
    IMPASSABLE_RIVER = 201
    IMPASSABLE_POND = 202
    IMPASSABLE_MOUNTAIN = 203
    IMPASSABLE_LAST = 204

    OBJECTS_FIRST = 301
    OBJECTS_SIGN = 301
    OBJECTS_HILL = 302
    OBJECTS_ROCKS = 303
    OBJECTS_TREES = 304
    OBJECTS_MILL = 305
    OBJECTS_HOUSES = 306
    OBJECTS_ROAD_DIRT = 307
    OBJECTS_ROAD_STONE = 308
    OBJECTS_CAMP = 309
    OBJECTS_FARM = 310
    OBJECTS_LAST = 311

    BUILDINGS_FIRST = 401
    BUILDINGS_ASSIGNED = 401
    BUILDINGS_WALL = 402
    BUILDINGS_CITY = 403
    BUILDINGS_LAST = 404

    UNITS_FIRST = 501
    UNITS_UNIT = 501
    UNITS_LAST = 502

    MAX_VALUE = 1000

    @staticmethod
    def fromName(name: str) -> CellValue:
        try:
            return CellValue[name]
        except KeyError:
            raise ValueError(f"Invalid cell value name {name}")

    def toName(self) -> str:
        return self.name


CellValueRanges = {
    "ground": (CellValue.GROUND_FIRST, CellValue.GROUND_LAST),
    "impassable": (CellValue.IMPASSABLE_FIRST, CellValue.IMPASSABLE_LAST),
    "objects": (CellValue.OBJECTS_FIRST, CellValue.OBJECTS_LAST),
    "buildings": (CellValue.BUILDINGS_FIRST, CellValue.BUILDINGS_LAST),
    "units": (CellValue.UNITS_FIRST, CellValue.UNITS_LAST),
}


def checkCellValue(layer: str, value: CellValue):
    if layer != "ground" and value == CellValue.NONE:
        return True
    valueRange = CellValueRanges[layer]
    return valueRange[0] <= value < valueRange[1]


def getCellValues(layer: str) -> List[int]:
    values: List[int] = []
    if layer != "ground":
        values.append(CellValue.NONE)
    for value in range(*CellValueRanges[layer]):
        values.append(value)
    return values