from __future__ import annotations

from enum import IntEnum, auto
from typing import Union, List

from .Resource import Resource

ItemPropertyValue = Union[str, int, List[Resource]]


class ItemProperty(IntEnum):
    LIFE_POINTS = 1
    MAX_LIFE_POINTS = auto()
    MOVE_POINTS = auto()
    MAX_MOVE_POINTS = auto()
    ACTION_POINTS = auto()
    MAX_ACTION_POINTS = auto()

    MELEE_ATTACK = auto()
    MELEE_DEFENSE = auto()
    MELEE_RANGE = auto()

    BOW_ATTACK = auto()
    BOW_DEFENSE = auto()
    BOW_RANGE = auto()

    MOUNT_ATTACK = auto()
    MOUNT_DEFENSE = auto()
    MOUNT_RANGE = auto()

    SIEGE_ATTACK = auto()
    SIEGE_DEFENSE = auto()
    SIEGE_RANGE = auto()

    WALL_LEVEL = auto()
    WALL_LEVEL_MAX = auto()

    BASE_PRODUCTION = auto()
    MERCHANT_PRODUCTION = auto()
    GRANARY_FOOD = auto()
    GRANARY_FOOD_MAX = auto()
    GROWTH_POINTS = auto()
    GROWTH_POINTS_RATE = auto()
    GROWTH_POINTS_MIN = auto()
    GROWTH_POINTS_MAX = auto()


    @staticmethod
    def fromName(name: str) -> ItemProperty:
        if name not in itemPropertyName2Id:
            raise ValueError(f"Invalid item property name {name}")
        return itemPropertyName2Id[name]

    def toName(self) -> str:
        return itemPropertyId2Name[self]


itemPropertyId2Name = {}
itemPropertyName2Id = {}
for property in ItemProperty:
    itemPropertyId2Name[property.value] = property.name
    itemPropertyName2Id[property.name] = property

attackProperties = {
    ItemProperty.MELEE_ATTACK: {
        "defense": ItemProperty.MELEE_DEFENSE,
        "range": ItemProperty.MELEE_RANGE,
    },
    ItemProperty.BOW_ATTACK: {
        "defense": ItemProperty.BOW_DEFENSE,
        "range": ItemProperty.BOW_RANGE,
    },
    ItemProperty.MOUNT_ATTACK: {
        "defense": ItemProperty.MOUNT_DEFENSE,
        "range": ItemProperty.MOUNT_RANGE,
    },
    ItemProperty.SIEGE_ATTACK: {
        "defense": ItemProperty.SIEGE_DEFENSE,
        "range": ItemProperty.SIEGE_RANGE,
    },
}

