from __future__ import annotations

from typing import Tuple, Optional, cast

from core.constants import CellValue
from core.logic.Command import Command, WORLD_PRIORITY, WORLD_MAX_WIDTH
from core.logic.Logic import Logic
from core.state import City


class UnassignCitizen(Command):

    def __init__(self, cityCell: Tuple[int, int], cell: Tuple[int, int]):
        self.__cityCell = cityCell
        self.__cell = cell
        self.__city: Optional[City] = None

    def priority(self) -> int:
        return WORLD_PRIORITY + self.__cityCell[0] + self.__cityCell[1] * WORLD_MAX_WIDTH

    def check(self, logic: Logic) -> bool:
        world = logic.world

        cityCell = self.__cityCell
        if not world.contains(cityCell):
            return False
        item = world.buildings.getItem(cityCell)
        if item is None or not isinstance(item, City):
            return False
        self.__city = cast(City, item)
        if self.__city.playerId != logic.state.playerId:
            return False

        cell = self.__cell
        if not world.contains(cell):
            return False
        if self.__city.findWorker(cell) is None:
            return False

        return True

    def execute(self, logic: Logic):
        assert self.__city is not None

        buildings = logic.world.buildings
        buildings.setItem(self.__cell, CellValue.NONE, None)
        buildings.notifyCellChanged(self.__cell)

        self.__city.unassignCitizen(self.__cell)
        logic.state.notifyCitizenUnassigned(self.__cityCell, self.__cell)
        logic.state.notifyResourcesChanged()

