from __future__ import annotations

from typing import cast

from core.constants import CellValue, checkCellValue
from core.logic.Logic import Logic
from core.state import City, Citizen
from .SetLayerCell import SetLayerCell


class SetBuildingsCell(SetLayerCell):

    def check(self, logic: Logic) -> bool:
        value = self._value
        if not checkCellValue("buildings", value):
            return False
        cell = self._cell
        world = logic.world
        if not world.contains(cell):
            return False

        if value == CellValue.NONE or self._item is None:
            if world.buildings.getValue(cell) == CellValue.NONE:  # Value already removed
                return False
        else:
            buildings = world.buildings
            ground = world.ground

            def canBuild(coords):
                if not buildings.contains(coords):
                    return False
                if buildings.getValue(coords) != CellValue.NONE:  # Value already set
                    return False
                if ground.getValue(coords) == CellValue.GROUND_SEA:  # Can't build on the sea
                    return False

                # Impassable
                impassableLayer = world.impassable
                impassableValue = impassableLayer.getValue(coords)
                if impassableValue != CellValue.NONE:  # Can't build on mountains etc.
                    return False

                # Objects
                objectsLayer = world.objects
                objectsValue = objectsLayer.getValue(coords)
                if objectsValue != CellValue.NONE:  # Can't build on trees etc.
                    return False
                return True

            if value != CellValue.BUILDINGS_CITY:
                if not canBuild(cell):
                    return False
            else:
                x0, y0 = cell
                for y in range(y0, y0 + 2):
                    for x in range(x0, x0 + 2):
                        if not canBuild((x, y)):
                            return False
                for y in range(y0 - 1, y0 + 3):
                    for x in range(x0 - 1, x0 + 3):
                        if buildings.contains((x, y)) and \
                                buildings.getValue((x, y)) == CellValue.BUILDINGS_CITY:
                            return False
        return True

    def execute(self, logic: Logic):
        cell = self._cell
        value = self._value
        world = logic.world

        if value == CellValue.NONE \
                and world.buildings.getValue(cell) == CellValue.BUILDINGS_CITY:
            x0, y0 = cell
            for y in range(y0 - 1, y0 + 2):
                for x in range(x0 - 1, x0 + 2):
                    cell = x, y
                    if not world.contains(cell):
                        continue
                    if world.buildings.getValue(cell) == CellValue.BUILDINGS_CITY:
                        world.buildings.setItem(cell, value)
                        world.buildings.notifyCellChanged(cell)
        elif value != CellValue.BUILDINGS_CITY:
            world.buildings.setValue(cell, value)
            world.buildings.notifyCellChanged(cell)
        else:
            city = cast(City, self._item)
            city.clearCitizens()
            x0, y0 = cell
            for y in range(y0, y0 + 2):
                for x in range(x0, x0 + 2):
                    cell = x, y
                    world.buildings.setItem(cell, value, city)
                    world.buildings.notifyCellChanged(cell)
            citizenCount = logic.rules.computeCityzenCount(cell)
            print("count", citizenCount)
            for citizenIndex in range(citizenCount):
                city.addCitizen(Citizen())
