from typing import Dict, Any, Union, cast

from ..constants import Resource


class Player:

    def __init__(self, idOrDict: Union[int, Dict[str, Any]]):
        if type(idOrDict) == int:
            self.__id = idOrDict
            self.__resources: Dict[Resource, int] = {}
        elif type(idOrDict) == dict:
            self.takeData(idOrDict)
        else:
            raise ValueError("Invalid data type")

    def __eq__(self, other) -> bool:
        if not isinstance(other, Player):
            return False
        player = cast(Player, other)
        if self.__id != player.__id:
            return False
        if self.__resources != player.__resources:
            return False
        return True

    @property
    def id(self) -> int:
        return self.__id

    def getResource(self, resource: Resource) -> int:
        if resource not in self.__resources:
            return 0
        return self.__resources[resource]

    def setResource(self, resource: Resource, amount: int):
        if amount != 0:
            self.__resources[resource] = int(amount)
        elif resource in self.__resources:
            del self.__resources[resource]

    def addResource(self, resource: Resource, amount: int):
        if resource in self.__resources:
            amount = int(amount) + self.__resources[resource]
        self.setResource(resource, amount)

    def gatherData(self) -> Dict[str, Any]:
        return {
            "id": self.__id,
            "resources": {resource.toName(): amount for resource, amount in self.__resources.items()}
        }

    def takeData(self, data: Dict[str, Any]):
        self.__id = int(data["id"])
        self.__resources = {Resource.fromName(name): amount for name, amount in data["resources"].items()}
