from core.constants import Resource
from core.logic import Logic
from core.state import GameState, IGameStateListener
from .FrameComponent import FrameComponent
from ..control import Label
from ...theme.Theme import Theme
from ...tools import formatResourceBalance


class ResourcesFrame(FrameComponent, IGameStateListener):

    def __init__(self, theme: Theme, logic: Logic):
        super().__init__(theme)
        self.__logic = logic

        self.__playerLabel = Label(theme)
        self.addComponent(self.__playerLabel)
        self.__woodLabel = Label(theme, "0<wood>")
        self.addComponent(self.__woodLabel)
        self.__stoneLabel = Label(theme, "0<stone>")
        self.addComponent(self.__stoneLabel)
        self.__goldLabel = Label(theme, "0<gold>")
        self.addComponent(self.__goldLabel)

        self.__updateValues()

        # Listeners
        self.__logic.state.registerListener(self)

    def dispose(self):
        self.__logic.state.removeListener(self)

    # GameState listener

    def turnChanged(self, state: GameState):
        self.__updateValues()

    def resourcesChanged(self, state: GameState):
        self.__updateValues()

    def __updateValues(self):
        player = self.__logic.state.getPlayer()
        wood = player.getResource(Resource.WOOD)
        stone = player.getResource(Resource.STONE)
        gold = player.getResource(Resource.GOLD)
        production = self.__logic.rules.computePlayerProduction(player.id)["balance"]

        self.__playerLabel.setMessage(f'<flag player="{player.id}">')
        self.__woodLabel.setMessage(f"{wood: 4}({formatResourceBalance(Resource.WOOD, production)})<wood>")
        self.__stoneLabel.setMessage(f"{stone: 4}({formatResourceBalance(Resource.STONE, production)})<stone>")
        self.__goldLabel.setMessage(f"{gold: 4}({formatResourceBalance(Resource.GOLD, production)})<gold>")

        borderSize = self.theme.framePadding
        self.__playerLabel.moveRelativeTo("topLeft", self, "topLeft", borderSize=borderSize)
        self.__woodLabel.moveRelativeTo("topLeft", self.__playerLabel, "topRight", borderSize=borderSize)
        self.__stoneLabel.moveRelativeTo("topLeft", self.__woodLabel, "topRight", borderSize=borderSize)
        self.__goldLabel.moveRelativeTo("topLeft", self.__stoneLabel, "topRight", borderSize=borderSize)

        self.pack()



