from typing import Optional, Tuple

from pygame.surface import Surface

from core.logic import DistanceMap
from core.state import Layer, GameState
from tools.vector import vectorSubI, vectorSubDiv2I
from ui.layer.SelectionValue import SelectionValue
from .LayerComponent import LayerComponent
from .LayerRenderer import LayerRenderer
from ..text.token import TextStyle
from ...theme.Theme import Theme


class AnimationComponent(LayerComponent):
    def __init__(self, theme: Theme, state: GameState, layer: Layer):
        super().__init__(theme, layer, "selection")
        self.__state = state
        self.__textStyle = TextStyle("small", "red")

        self.__damageCell: Optional[Tuple[int, int]] = None
        self.__damage = 0
        self.__damageShift = 0

        self.__graveCell: Optional[Tuple[int, int]] = None
        self.__graveTime = 0

    def addDamage(self, cell: Tuple[int, int], damage: int):
        self.__damage = damage
        self.__damageCell = cell
        self.__damageShift = 0

    def addGrave(self, cell: Tuple[int, int]):
        self.__graveCell = cell
        self.__graveTime = 0

    def __renderDamage(self, surface: Surface, renderer: LayerRenderer):
        if self.__damageCell is None:
            return
        dest = renderer.computeDest(self.__damageCell)
        if dest is None:
            return
        x, y = dest
        tileWidth, tileHeight = self.tileset.tileSize
        textSurface = self.__textStyle.render(self.theme, f"-{self.__damage}")
        textSurface.set_colorkey((0, 0, 0))
        surface.blit(textSurface, (
            x + (tileWidth - textSurface.get_width()) // 2,
            y + (tileHeight - textSurface.get_height()) // 2 - self.__damageShift
        ))
        self.__damageShift += 1
        if self.__damageShift > 2 * tileHeight:
            self.__damageCell = None

    def __renderGrave(self, surface: Surface, renderer: LayerRenderer):
        if self.__graveCell is None:
            return
        dest = renderer.computeDest(self.__graveCell)
        if dest is None:
            return
        rect = self.tileset.getTileRect(SelectionValue.GRAVE)
        surface.blit(self.tileset.surface, dest, rect)
        self.__graveTime += 1
        if self.__graveTime > 100:
            self.__graveCell = None

    def render(self, surface: Surface):
        super().render(surface)
        renderer = self.createRenderer(surface)
        self.__renderGrave(surface, renderer)
        self.__renderDamage(surface, renderer)

