from typing import cast

from pygame.surface import Surface

from core.constants import CellValue, Direction
from core.state import World, City
from .LayerComponent import LayerComponent
from ...theme.Theme import Theme


class BuildingsComponent(LayerComponent):
    def __init__(self, theme: Theme, world: World):
        super().__init__(theme, world.buildings, "buildings")
        self.__buildings = world.buildings
        self.__showAssigned = False

    @property
    def showAssigned(self) -> bool:
        return self.__showAssigned

    @showAssigned.setter
    def showAssigned(self, value: bool):
        self.__showAssigned = value

    def render(self, surface: Surface):
        super().render(surface)
        tileset = self.tileset.surface
        tilesRects = self.tileset.getTilesRects()

        renderer = self.createRenderer(surface)
        cellsSlice = renderer.cellsSlice
        cellMinX, cellMinY, _, _ = renderer.cellsBox
        cells = self.__buildings.cells[cellsSlice]

        valid = cells == CellValue.BUILDINGS_CITY
        buildings = self.__buildings
        for dest, value, cell in renderer.cells(valid):
            rects = tilesRects[value]
            item = buildings.getItem(cell)
            city = cast(City, item)
            rectIndex = city.playerId * 12
            leftValue = buildings.getValue(cell, Direction.LEFT)
            topValue = buildings.getValue(cell, Direction.TOP)
            if leftValue != CellValue.BUILDINGS_CITY:
                if topValue != CellValue.BUILDINGS_CITY:
                    surface.blit(tileset, dest, rects[rectIndex + 0])
                else:
                    surface.blit(tileset, dest, rects[rectIndex + 2])
            else:
                if topValue != CellValue.BUILDINGS_CITY:
                    surface.blit(tileset, dest, rects[rectIndex + 1])
                else:
                    surface.blit(tileset, dest, rects[rectIndex + 3])

        valid = cells != CellValue.NONE
        valid &= cells != CellValue.BUILDINGS_CITY
        if not self.__showAssigned:
            valid &= cells != CellValue.BUILDINGS_ASSIGNED
        for dest, value, _ in renderer.cells(valid):
            rects = tilesRects[value]
            surface.blit(tileset, dest, rects[0])
