from typing import Tuple, Dict

from core.constants import Resource
from core.state import World
from tools.vector import vectorMulI, vectorSubHalfI, vectorSubI, vectorClampI
from ui.theme import Theme


def centerCellView(theme: Theme, world: World, cell: Tuple[int, int]) -> Tuple[int, int]:
    """
    Compute the top left world coordinates such as the cell is centered on screen.
    """
    tileSize = theme.defaulTileSize
    # Convert cell coordinates into pixel coordinates
    view = vectorMulI(cell, tileSize)
    # Center so the cursor in the middle of the view
    view = vectorSubHalfI(view, theme.viewSize)
    worldSize = vectorMulI(world.size, tileSize)
    maxView = vectorSubI(worldSize, theme.viewSize)
    return vectorClampI(view, 0, maxView)


def formatResourceBalance(resource: Resource, resources: Dict[Resource, int]) -> str:
    if resource not in resources:
        return "0"
    count = resources[resource]
    if count > 0:
        return f"+{count}"
    else:
        return f"{count}"